/*
 * querydevice function 
 * by Kirby Kuehl (kkuehl@cisco.com)
 * This function accepts the following:
 * router       - File containing results from queryrouter function
 *                or parsed results from nmap scan
 * wwwauth      - HTTP Basic Authentication (To be passed to querydevice function)
 * type         - Scan Type  AIRONET, POLYCOM, or NMAP
 */

#include "aptools.h"
void timedout(int signo);

void aironetquery(char *iptoscan, char *wwwauth, GtkWidget *text)
{
   struct hostent     *host;
   struct sockaddr_in address;
   struct timeval tv;

   int sockfd = 0, count = 0, start = 0, numofchecks = 0;
   char sendbuff[SOCKBUFF], recvbuff[SOCKBUFF];
 
   static  char *uri[] = {
                          "HEAD / HTTP/1.0\r\n",
                          "GET /SetWep_Keys.shm HTTP/1.0\r\n",
                          "GET /SetSNMP.shm HTTP/1.0\r\n",
                          "GET /SetUserManager.shm HTTP/1.0\r\n ",
                          "GET /SetHwPC4800.shm HTTP/1.0\r\n",
                          "GET /SetIdentPC4800.shm HTTP/1.0\r\n"
   };

   struct in_addr     inaddr;
  
   memset(&address,0,sizeof(address));
   if(inet_pton(AF_INET, iptoscan, &inaddr))
      host = gethostbyaddr((char *) &inaddr, sizeof(inaddr),AF_INET);
   else
      host = gethostbyname(iptoscan);
   if(!host)
   {
      fprintf(stdout,  "--------------------------------------------------------\n");
      fprintf(stdout,  "Unable to resolve host %s\n",iptoscan);
   }
   else
      memcpy(&address.sin_addr, host->h_addr_list[0], sizeof(address.sin_addr));

   fprintf(stdout,  "--------------------------------------------------------\n");
      
   if(host)
   {
      fprintf(stdout,  "Implementing Aironet Checks against\n%s %s\n",host==NULL ? "unknown" : host->h_name,iptoscan);
      start = 0;
      numofchecks = 5;
   }
   
   for(count = start;count <= numofchecks; count++)
   {
      tv.tv_sec  = SNDRCVTIMEOUT;
      tv.tv_usec = SNDRCVTIMEOUT;
      if((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
      {
         fprintf(stderr, "Unable to initialize socket");
         perror(iptoscan);
         continue;
      }
      address.sin_family = AF_INET;
      address.sin_port   = htons(80);
      alarm(0);
      signal(SIGALRM, timedout);
      alarm(CONNECTTIMEOUT);
      
      if((connect(sockfd, (struct sockaddr *)&address, sizeof(address)) == -1))
      {
         if(errno==EINTR)
            errno=ETIMEDOUT;
         fprintf(stdout,  "%s is a client.\n",iptoscan);
         close(sockfd);
         break;
      }
      alarm(0);
      signal(SIGALRM,SIG_DFL);
      
      memset(sendbuff,0,SOCKBUFF);
      memset(recvbuff,0,SOCKBUFF);

      snprintf(sendbuff,sizeof(sendbuff),
               "%sReferer: http://%s/\r\n%sHost: %s\r\n%s",
               uri[count],iptoscan,AGENT,iptoscan,
               needauth==1 ? wwwauth : "\r\r\n");

      if(send(sockfd,sendbuff,strlen(sendbuff),0) == -1)
      {
         fprintf(stderr, "Error sending request.\n");
         close(sockfd);
         return;
      }
      else
         gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL,
                       sendbuff, -1);
 
      while(recv(sockfd,recvbuff,SOCKBUFF,0)>0)
      {
         if(strstr(recvbuff, "401 Unauthorized"))
         {
            /* Only send http basic authentication (base 64 encoded userid:password)
             * if we receive a 401 Unauthorized                                
             */
            fprintf(stdout,  "Authorization required %s\n",uri[count]);
            needauth = 1;
            aironetquery(iptoscan, wwwauth,text);
            close(sockfd);
            break;
         }

         if(strstr(recvbuff,"404 Not Found"))
         {
            fprintf(stdout,  "404 Not Found for %s\n",uri[count]);
            close(sockfd);
            break;
         }

         if(strstr(recvbuff,"Server: Microsoft-IIS"))
         {
            fprintf(stdout, "Aironet Client running IIS\n");
            close(sockfd);
            break;
         }
                
         if(strstr(recvbuff,"Server: TransMedia"))
         {
            fprintf(stdout, "False Positive: Cisco MGX Media Gateway\n");
            close(sockfd);
            break;
         }
        
         if(strstr(recvbuff, "Server: GoAhead-Webs"))
         {
            fprintf(stdout, "False Positive: RadVision ViaIP Device\n");
            close(sockfd);
            break;
         }
         
         if(strstr(recvbuff, "<option value=\"FULL\" selected>"))
         {
            fprintf(stdout,  "Full Encryption selected: PASSED\n");
         }

         if(strstr(recvbuff, "<option value=\"13\" selected>128 bit"))
         {
            fprintf(stdout,  "Key Size 128 bit: PASSED\n");
            }

            if(strstr(recvbuff,"value=\"F\" checked name=\"radio_allowBrowseWithoutLogin"))
            {
               fprintf(stdout,  "Allow Read-Only Browsing without Login: PASSED\n");
            }

            if(strstr(recvbuff,"value=\"T\" checked name=\"radio_allowBrowseWithoutLogin"))
            {
               fprintf(stdout,  "Allow Read-Only Browsing without Login: FAILED\n");
            }

            if(strstr(recvbuff,"value=\"F\" checked name=\"radio_enableSNMP"))
            {
               fprintf(stdout,  "SNMP disabled: FAILED\n");
            }

            if(strstr(recvbuff,"value=\"T\" checked name=\"radio_enableSNMP"))
            {
               fprintf(stdout,  "SNMP enabled: FAILED\n");
            }

            if(strstr(recvbuff,"value=\"T\" checked name=\"radio_awcDot11AllowAssocBrodcastSSID"))
            {
               fprintf(stdout,  "Allow Broadcast SSID to Associate: FAILED\n");
            }

            if(strstr(recvbuff,"value=\"F\" checked name=\"radio_awcDot11AllowAssocBroadcastSSID"))
            {
               fprintf(stdout,  "Allow Broadcast SSID to Associate: PASSED\n");
            }

            if(strstr(recvbuff,"name=\"checkbox_dot11AuthenticationAlgorithmsEnable.2.1\" value=\"true\" checked"))
            {
               fprintf(stdout,  "Accept Authentication Types (open): PASSED\n");
            }

            if(strstr(recvbuff,"value=\"tsunami\" name=\"text_dot11DesiredSSID"))
            {
               fprintf(stdout,  "Service Set ID (SSID): FAILED\n");
            }

            if(strstr(recvbuff,"configuration radio ssid \"tsunami\""))
            {
               fprintf(stdout,  "Service Set ID (SSID): FAILED\n");
            }
         }            /* While receiving data from the buffer    */
      }               /* While checking each URL                 */
      close(sockfd);  /* Close socket                            */
   return;
}

