#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <stdlib.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "aptools.h"

char * base64_encode_line(const char *s);

void
on_Clear_clicked                       (GtkButton       *button,
                                        gpointer         user_data)
{
   guint length = 0;
   GtkWidget * text = lookup_widget(GTK_WIDGET(button),"Output");
   length = gtk_text_get_length (GTK_TEXT(text));
   gtk_text_backward_delete(GTK_TEXT(text),length);
}


void
on_Scan_clicked                        (GtkButton       *button,
                                        gpointer         user_data)
{
   char buffer[1024], httpuserid[1024], httppassword[1024], command[1024];
   gchar *string;
   struct in_addr inaddr;
   struct hostent *host;
   FILE *input;
   char ipaddress[MAXHOSTNAMELEN];
   char *t1, *t2, *wwwauth;
   int type = AIRONET;

   GtkWidget * tacacs_password = lookup_widget(GTK_WIDGET(button),
		                               "TACACS_Password");
   GtkWidget * tacacs_userid   = lookup_widget(GTK_WIDGET(button),
		                               "TACACS_Userid");
   GtkWidget * http_password   = lookup_widget(GTK_WIDGET(button),
		                               "HTTP_Password");
   GtkWidget * http_userid     = lookup_widget(GTK_WIDGET(button),
		                               "HTTP_Userid");
   GtkWidget * text            = lookup_widget(GTK_WIDGET(button),
		                               "Output");
   GtkWidget * input_type      = lookup_widget(GTK_WIDGET(button),
		                               "combo1");
   GtkWidget * ap_type         = lookup_widget(GTK_WIDGET(button),
		                               "combo2");
   GtkWidget * host_file       = lookup_widget(GTK_WIDGET(button),
		                               "Host_Filename");
   /***
    * Determine if entry is a valid hostname or a filename.
    */

   if(inet_pton(AF_INET,gtk_entry_get_text(GTK_ENTRY(host_file)), &inaddr))
      host = gethostbyaddr((char *) &inaddr, sizeof(inaddr),AF_INET);
   else
      host = gethostbyname(gtk_entry_get_text(GTK_ENTRY(host_file)));

   if(host)
   {
     snprintf(buffer,1024,"%s is a valid hostname\n",
		     gtk_entry_get_text(GTK_ENTRY(host_file)));
     snprintf(ipaddress,MAXHOSTNAMELEN,"%s",
		     gtk_entry_get_text(GTK_ENTRY(host_file)));
     gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL,
        	       buffer, -1);
   }
   else
   {
     string = gtk_entry_get_text(GTK_ENTRY(host_file));
     if((input = fopen(string,"r")) == NULL)
     {
        snprintf(buffer,1024,"%s is not a valid filename\n",
		     gtk_entry_get_text(GTK_ENTRY(host_file)));
       gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL,
        	       buffer, -1);
     }
     else
     {
        while(fgets(ipaddress, sizeof(ipaddress),input) != NULL)
	{
           while(strlen(ipaddress) && (ipaddress[strlen(ipaddress) -1] == '\n' \
                 || ipaddress[strlen(ipaddress) -1] == '\r'))
              ipaddress[strlen(ipaddress) -1] = '\0';
              gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL,
        	              ipaddress, -1);
	}
     }
   }
  
   /***
   * Base64 Encoding for Basic HTTP Authentication. Based on code from wget
   */

   snprintf(httpuserid,1024,"%s",gtk_entry_get_text(GTK_ENTRY(http_userid)));
   snprintf(httppassword,1024,"%s",gtk_entry_get_text(GTK_ENTRY(http_password))); 
   if((httpuserid[0] != '\0') || (httppassword[0] != '\0'))
   {
      t1 = (char *)malloc(strlen(httpuserid) * strlen(httppassword) + 1);
      snprintf(t1,1024,"%s:%s",httpuserid[0]=='\0' ? "" : httpuserid,
		               httppassword[0]=='\0' ? "" : httppassword);
      t2 = base64_encode_line(t1);
      free(t1);
      wwwauth = (char *)malloc(strlen(t2)+27);
      snprintf(wwwauth,1024,"Authorization: Basic %s\r\n\r\n",t2);
      gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL,
                      wwwauth, -1);
   }
   else
   {
      wwwauth = (char *)malloc(1);
      snprintf(wwwauth,1,"\0");
   }
 
   snprintf(buffer,1024,"tacacs userid = %s tacacs_password = %s http userid = %s http password = %s\n",
		   gtk_entry_get_text(GTK_ENTRY(tacacs_userid)),
		   gtk_entry_get_text(GTK_ENTRY(tacacs_password)),
		   gtk_entry_get_text(GTK_ENTRY(http_userid)),
                   gtk_entry_get_text(GTK_ENTRY(http_password)));
   gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, buffer, -1);

   /* Determine Scan Type */ 
   string = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (input_type)->entry));
   if(strncmp(string,"Single Aironet",14)==0)
      type = AIRONET;
   if(strncmp(string,"List of Aironets",16)==0)
      type = AIRONETLIST;
   if(strncmp(string,"Single Router",13)==0)
      type = ROUTER;
   if(strncmp(string,"List of Routers",15)==0)
      type = ROUTERLIST;
   if(strncmp(string,"Single Switch",13)==0)
      type = SWITCH;
   if(strncmp(string,"List of Switches",16)==0)
      type = SWITCHLIST;
   gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, string, -1); 
  
   /* Determine Access Point Type */
   string = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (ap_type)->entry));
   gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, string, -1);
   if(strncmp(string,"Aironet",7)==0)
      strncpy(command,"show ip arp | include 0040.96\r\n",sizeof(command));

   if(type == AIRONET)
      aironetquery(ipaddress, wwwauth, text); 
}


void
on_Exit_clicked                        (GtkButton       *button,
                                        gpointer         user_data)
{
   gtk_main_quit();
}

