/*
 * queryrouter function 
 * by Kirby Kuehl (kkuehl@cisco.com)
 * This function accepts the following:
 * iptoscan - IP Address of a router
 * username     - Valid Userid for router
 * password     - Valid Password for router
 * wwwauth      - HTTP Basic Authentication (To be passed to querydevice function)
 */

#include "aptools.h"
void timedout(int signo);
void querysrouter(char *iptoscan, char *username, char *password, char *wwwauth, char *command);

char queryswitch(char *iptoscan, char *username, char *password, char *wwwauth)
{
   FILE *routeroutput;
   int sockfd = 0;
   char routerresults[100];
   struct hostent     *host;
   struct sockaddr_in address;
   #ifdef WIN32
   u_long res;
   WSADATA WSAData;
   if(WSAStartup (MAKEWORD(1,1), &WSAData) != 0)
   {
      fprintf(stderr,"WSAStartup failed!\n");
      WSACleanup();
      return(-1);
   }
   #else
   struct in_addr     inaddr;
   #endif
   struct timeval     tv;
   
   char sendbuff[SOCKBUFF], recvbuff[SOCKBUFF];

   snprintf(routerresults,100,"%s.txt",iptoscan);
   if((routeroutput = fopen(routerresults, "a")) == NULL)
   {
      fprintf(stderr, "Unable to open %s\n",routerresults);
      return(-1);
   }

   fprintf(stdout, "Writing Devices to %s\n",routerresults);
   memset(&address,0,sizeof(address));

   #ifndef WIN32
   if(inet_pton(AF_INET, iptoscan, &inaddr))
      host = gethostbyaddr((char *) &inaddr, sizeof(inaddr),AF_INET);
   #else
   if((res = inet_addr (iptoscan)) != INADDR_NONE )
      host = gethostbyaddr ((char*) &res, sizeof(res),AF_INET);
   #endif
      host = gethostbyname(iptoscan);

   if(!host)
   {
      fprintf(stdout, "--------------------------------------------------------\n");
      fprintf(stderr, "Unable to resolve host %s\n",iptoscan);
   }
   else
      memcpy(&address.sin_addr, host->h_addr_list[0], sizeof(address.sin_addr));

   fprintf(stdout, "--------------------------------------------------------\n");
   fprintf(stdout, "Locating Devices Connected to %s\n",iptoscan);

   if((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
   {
      fprintf(stderr, "Unable to initialize socket");
      perror(iptoscan);
      fclose(routeroutput);
      return(-1);
   }

   tv.tv_sec  = SNDRCVTIMEOUT;
   tv.tv_usec = SNDRCVTIMEOUT;
   #if defined (_BSD)
   if(setsockopt(sockfd,SOL_SOCKET,SO_RCVTIMEO,&tv,sizeof(tv))!=0)
      perror("setsockopt");
   if(setsockopt(sockfd,SOL_SOCKET,SO_SNDTIMEO,&tv,sizeof(tv))!=0)
      perror("setsockopt");
   if(setsockopt(sockfd,SOL_SOCKET,SO_RCVLOWAT,(void *) &wait,sizeof(wait))!=0)
      perror("setsockopt");
   #endif

   #ifndef WIN32
   signal(SIGALRM, timedout);
   alarm(CONNECTTIMEOUT);
   #endif
   address.sin_family = AF_INET;
   address.sin_port   = htons(23);
   if((connect(sockfd, (struct sockaddr *)&address, sizeof(address)) == -1))
   {
       #ifndef WIN32
       if(errno==EINTR)
         errno=ETIMEDOUT;
       #endif
      fprintf(stderr, "Error establishing connection to port 23 on\n");
      perror(iptoscan);
      fclose(routeroutput);
      close(sockfd);
      return(-1);
   }
   #ifndef WIN32
   alarm(0);
   signal(SIGALRM,SIG_DFL);
   #endif
   memset(recvbuff,0,SOCKBUFF);
   while(recv(sockfd, recvbuff,SOCKBUFF,0)>0)
   {
      #ifdef DEBUG
      fprintf(stdout,"%s",recvbuff);
      #endif
      if(strstr(recvbuff,"sername:"))
         break;
      memset(recvbuff,0,SOCKBUFF);
   }
   memset(sendbuff,0,SOCKBUFF);
   snprintf(sendbuff,SOCKBUFF,"%s\r\n",username);
   if(send(sockfd,sendbuff,strlen(sendbuff),0) == -1)
   {
      fprintf(stderr, "Error Sending Username\n");
      return(-1);
   }

   memset(recvbuff,0,SOCKBUFF);
   while(recv(sockfd, recvbuff,SOCKBUFF,0)>0)
   {
      #ifdef DEBUG
      fprintf(stdout,"%s",recvbuff);
      #endif
      if(strstr(recvbuff,"assword:"))
         break;
      memset(recvbuff,0,SOCKBUFF);
   }

   snprintf(sendbuff,SOCKBUFF,"%s\r\n",password);
   if(send(sockfd,sendbuff,strlen(sendbuff),0) == -1)
   {
      fprintf(stderr, "Error Sending Password\n");
      return(-1);
   }
   else
      fprintf(stdout, "********\n");

   memset(recvbuff,0,SOCKBUFF);
   while(recv(sockfd,recvbuff, SOCKBUFF,0)>0)
   {
      #ifdef DEBUG
      fprintf(stdout,"%s",recvbuff);
      #endif
      if(strstr(recvbuff,"Authentication failed"))
      {
         fprintf(stderr,"Error: Invalid Password\n");
         return(-1);
      } 
      if(strstr(recvbuff,">"))
         break;
      memset(recvbuff,0,SOCKBUFF);
   }

   memset(sendbuff,0,SOCKBUFF);
   snprintf(sendbuff,15,"set length 0\r\n");
   #ifdef DEBUG
   fprintf(stdout,"Sendbuff[%s]",sendbuff);
   #endif
   if((send(sockfd,sendbuff,strlen(sendbuff),0)) == -1)
   {
      fprintf(stderr,"Error: Unable to set terminal length to 0\n");
      return(-1);
   }
               
   memset(recvbuff,0,SOCKBUFF);
   while(recv(sockfd,recvbuff, SOCKBUFF,0)>0)
   {
      if(strstr(recvbuff,">"))
         break;
      memset(recvbuff,0,SOCKBUFF);
   }
   
   snprintf(sendbuff,19,"show cam dynamic\r\n");
   if((send(sockfd,sendbuff,strlen(sendbuff),0)) == -1)
   {
        fprintf(stderr,"Error sending :%s\n",sendbuff);
        return(-1);
   }
   memset(recvbuff,0,SOCKBUFF);

   while(recv(sockfd,recvbuff, SOCKBUFF,0)>0)
   { 
       fprintf(stdout,"%s",recvbuff);
       fprintf(routeroutput,"%s",recvbuff); 
       if(strstr(recvbuff,">"))
          break;
       memset(recvbuff,0,SOCKBUFF);
   }
      
   snprintf(sendbuff,21,"show cdp neighbors\r\n");
   if((send(sockfd,sendbuff,strlen(sendbuff),0)) == -1)
   {
      fprintf(stderr,"Error sending %s\n",sendbuff);
      return(-1);
   }
   memset(recvbuff,0,SOCKBUFF); 
   while(recv(sockfd,recvbuff,SOCKBUFF,0)>0)
   {
      fprintf(stdout,"%s",recvbuff);
      fprintf(routeroutput,"%s",recvbuff); 
      if(strstr(recvbuff,">"))
         break;
      memset(recvbuff,0,SOCKBUFF);
   }
  
   snprintf(sendbuff,19,"show cam dynamic\r\n");
   if((send(sockfd,sendbuff,strlen(sendbuff),0)) == -1)
   {
      fprintf(stderr,"Error sending %s\n",sendbuff);
      return(-1);
   }
   memset(recvbuff,0,SOCKBUFF);
   while(recv(sockfd,recvbuff,SOCKBUFF,0)>0)
   {
      fprintf(stdout,"%s",recvbuff);
      fprintf(routeroutput,"%s",recvbuff);
      if(strstr(recvbuff,">"))
         break;
      memset(recvbuff,0,SOCKBUFF);
   }


  snprintf(sendbuff,7,"exit\r\n");
  if((send(sockfd,sendbuff,strlen(sendbuff),0)) == -1)
  {
      fprintf(stderr, "Error logging out\n");
      return(-1);
  }

  fclose(routeroutput);
  close(sockfd);
  #ifdef WIN32
  WSACleanup();
  #endif
  return(0);
}

